@@ -32,9 +32,14 @@ class Auth(auth.SASLAuth):
 
     def __init__(self, user, password, ssl=True, method=SASLMethod.SCRAM_SHA_512, **kwargs):
         super().__init__(user, password, ssl=ssl, method=method, **kwargs)
+        self._username = user
+
+    @property
+    def username(self):
+        return self._username
 
 
-def load_auth(config_file=configure.get_config_path()):
+def load_auth(config_file=None):
     """Configures an Auth instance given a configuration file.
 
     Args:
@@ -48,8 +53,16 @@ def load_auth(config_file=configure.get_config_path()):
         KeyError: An error occurred parsing the configuration file.
 
     """
+    if config_file is None:
+        config_file = configure.get_config_path()
     if not os.path.exists(config_file):
         raise FileNotFoundError(errno.ENOENT, os.strerror(errno.ENOENT), config_file)
+    # check that the config file has suitable permissions
+    config_props = os.stat(config_file)
+    dangerous_perm_mask = 0o7077
+    if (config_props.st_mode & dangerous_perm_mask) != 0:
+        raise RuntimeError(f"{config_file} has unsafe permissions: {config_props.st_mode:o}\n"
+                           "Please correct it to 0600")
 
     # load config
     with open(config_file, "r") as f:
