@@ -1,6 +1,5 @@
 from dataclasses import dataclass
 from functools import lru_cache
-import getpass
 from enum import Enum
 import json
 import logging
@@ -31,11 +30,12 @@ class Stream(object):
     stream connection is opened, it will use defaults specified here.
 
     Args:
-        auth: A `bool` or `Auth` instance. Defaults to loading from `auth.load_auth()`
-            if set to True. To disable authentication, set to False.
+        auth: A `bool` or :class:`Auth <hop.auth.Auth>` instance. Defaults to
+            loading from :meth:`auth.load_auth <hop.auth.load_auth>` if set to
+            True. To disable authentication, set to False.
         start_at: The message offset to start at in read mode. Defaults to LATEST.
         persist: Whether to listen to new messages forever or stop
-             when EOS is received in read mode. Defaults to False.
+            when EOS is received in read mode. Defaults to False.
 
     """
 
@@ -73,13 +73,11 @@ class Stream(object):
 
         Args:
             url: Sets the broker URL to connect to.
-
-        Kwargs:
             mode: Read ('r') or write ('w') from the stream.
 
         Returns:
-            An open connection to the client, either an adc Producer instance
-            in write mode or an adc Consumer instance in read mode.
+            An open connection to the client, either a :class:`Producer` instance
+            in write mode or a :class:`Consumer` instance in read mode.
 
         Raises:
             ValueError: If the mode is not set to read/write or if more than
@@ -90,17 +88,20 @@ class Stream(object):
         logger.debug("connecting to addresses=%s  group_id=%s  topics=%s",
                      broker_addresses, group_id, topics)
 
+        if topics is None:
+            raise ValueError("no topic(s) specified in kafka URL")
         if mode == "w":
             if len(topics) != 1:
                 raise ValueError("must specify exactly one topic in write mode")
             if group_id is not None:
                 warnings.warn("group ID has no effect when opening a stream in write mode")
-            return _Producer(broker_addresses, topics[0], auth=self.auth)
+            return Producer(broker_addresses, topics[0], auth=self.auth)
         elif mode == "r":
             if group_id is None:
-                group_id = _generate_group_id(10)
+                username = self.auth.username if hasattr(self.auth, "username") else None
+                group_id = _generate_group_id(username, 10)
                 logger.info(f"group ID not specified, generating a random group ID: {group_id}")
-            return _Consumer(
+            return Consumer(
                 group_id,
                 broker_addresses,
                 topics,
@@ -128,10 +129,13 @@ def _load_deserializer_plugins():
         manager.load_setuptools_entrypoints("hop_plugin")
     except Exception:
         logger.warning(
-            "could not load external message plugins as one or more plugins "
+            "Could not load external message plugins as one or more plugins "
             "generated errors upon import. to fix this issue, uninstall or fix "
             "any problematic plugins that are currently installed."
         )
+        import sys
+        import traceback
+        traceback.print_exc(file=sys.stderr)
 
     # add all registered plugins to registry
     registered = {}
@@ -140,7 +144,7 @@ def _load_deserializer_plugins():
             plugin_name = name.upper()
             if plugin_name in registered:
                 logger.warning(
-                    f"identified duplicate message plugin {plugin_name} registered under "
+                    f"Identified duplicate message plugin {plugin_name} registered under "
                     "the same name. this may cause unexpected behavior when using this "
                     "message format."
                 )
@@ -195,11 +199,12 @@ Deserializer = Enum(
 )
 
 
-def _generate_group_id(n):
+def _generate_group_id(user, n):
     """Generate a random Kafka group ID.
 
     Args:
-        n: Length of randomly generated string.
+        user: Username associated with the credential being used
+        n: Length of randomly generated string suffix.
 
     Returns:
         The generated group ID.
@@ -207,7 +212,9 @@ def _generate_group_id(n):
     """
     alphanum = string.ascii_uppercase + string.digits
     rand_str = ''.join(random.SystemRandom().choice(alphanum) for _ in range(n))
-    return '-'.join((getpass.getuser(), rand_str))
+    if user is None:
+        return rand_str
+    return '-'.join((user, rand_str))
 
 
 @dataclass(frozen=True)
@@ -235,8 +242,32 @@ class Metadata:
         )
 
 
-class _Consumer:
+class Consumer:
+    """
+    An event stream opened for reading one or more topics.
+    Instances of this class should be obtained from :meth:`Stream.open`.
+    """
+
     def __init__(self, group_id, broker_addresses, topics, **kwargs):
+        """
+        Args:
+            group_id: The Kafka consumer group to join for reading messages.
+            broker_addresses: The list of bootstrap Kafka broker URLs.
+            topics: The list of names of topics to which to subscribe.
+            read_forever: If true, keep the stream open to wait for more messages
+                after reading the last currently available message.
+            start_at: The position in the topic stream at which to start
+                reading, specified as a StartPosition object.
+            auth: An adc.auth.SASLAuth object specifying client authentication
+                to use.
+            error_callback: A callback which will be called with any
+                confluent_kafka.KafkaError objects produced representing internal
+                Kafka errors.
+            offset_commit_interval: A datetime.timedelta specifying how often to
+                report progress to Kafka.
+
+        :meta private:
+        """
         self._consumer = consumer.Consumer(consumer.ConsumerConfig(
             broker_urls=broker_addresses,
             group_id=group_id,
@@ -252,19 +283,26 @@ class _Consumer:
             metadata: Whether to receive message metadata alongside messages.
             autocommit: Whether messages are automatically marked as handled
                 via `mark_done` when the next message is yielded. Defaults to True.
-
+            batch_size: The number of messages to request from Kafka at a time.
+                Lower numbers can give lower latency, while higher numbers will
+                be more efficient, but may add latency.
+            batch_timeout: The period of time to wait to get a full batch of
+                messages from Kafka. Similar to batch_size, lower numbers can
+                reduce latency while higher numbers can be more efficient at the
+                cost of greater latency.
+                If specified, this argument should be a datetime.timedelta
+                object.
         """
         for message in self._consumer.stream(autocommit=autocommit, **kwargs):
-            yield self.unpack(message, metadata=metadata)
+            yield self._unpack(message, metadata=metadata)
 
     @staticmethod
-    def unpack(message, metadata=False):
+    def _unpack(message, metadata=False):
         """Deserialize and unpack messages.
 
         Args:
             message: The message to deserialize and unpack.
             metadata: Whether to receive message metadata alongside messages.
-
         """
         payload = json.loads(message.value().decode("utf-8"))
         payload = Deserializer.deserialize(payload)
@@ -298,8 +336,30 @@ class _Consumer:
         self.close()
 
 
-class _Producer:
+class Producer:
+    """
+    An event stream opened for writing to a topic.
+    Instances of this class should be obtained from :meth:`Stream.open`.
+    """
+
     def __init__(self, broker_addresses, topic, **kwargs):
+        """
+        Args:
+            broker_addresses: The list of bootstrap Kafka broker URLs.
+            topic: The name of the topic to which to write.
+            auth: An adc.auth.SASLAuth object specifying client authentication
+                to use.
+            delivery_callback: A callback which will be called when each message
+                is either delivered or permenantly fails to be delivered.
+            error_callback: A callback which will be called with any
+                confluent_kafka.KafkaError objects produced representing internal
+                Kafka errors.
+            produce_timeout: A datetime.timedelta object specifying the maximum
+                time to wait for a message to be sent to Kafka. If zero, sending
+                will never time out.
+
+        :meta private:
+        """
         self._producer = producer.Producer(producer.ProducerConfig(
             broker_urls=broker_addresses,
             topic=topic,
@@ -314,21 +374,27 @@ class _Producer:
             message: The message to write.
 
         """
-        self._producer.write(self.pack(message))
+        self._producer.write(self._pack(message))
 
     @staticmethod
-    def pack(message):
+    def _pack(message):
         """Pack and serialize messages.
 
         Args:
             message: The message to pack and serialize.
 
+        :meta private:
         """
         try:
             payload = message.serialize()
         except AttributeError:
             payload = {"format": "blob", "content": message}
-        return json.dumps(payload).encode("utf-8")
+        try:
+            return json.dumps(payload).encode("utf-8")
+        except TypeError:
+            raise TypeError("Unable to pack a message of type "
+                            + message.__class__.__name__
+                            + " which cannot be serialized to JSON")
 
     def close(self):
         """Wait for enqueued messages to be written and shut down.
