@@ -1,13 +1,12 @@
 from collections import defaultdict
-from functools import partialmethod
+from contextlib import contextmanager
 import io
+import os
+import stat
 from unittest.mock import MagicMock
 
 import pytest
 
-from adc.consumer import Consumer
-from adc.producer import Producer
-
 from hop import models
 from hop.io import StartPosition
 
@@ -185,12 +184,18 @@ class MockBroker:
     """
 
     def __init__(self):
+        self.reset()
+
+    def reset(self):
         self._messages = defaultdict(list)
         self._offsets = defaultdict(dict)
 
     def write(self, topic, msg):
         self._messages[topic].append(msg)
 
+    def has_message(self, topic, message):
+        return message in self._messages[topic]
+
     def read(self, topic, groupid, start_at=StartPosition.EARLIEST, **kwargs):
         if topic not in self._offsets or groupid not in self._offsets[topic]:
             if start_at == StartPosition.EARLIEST:
@@ -200,11 +205,10 @@ class MockBroker:
 
         try:
             offset = self._offsets[topic][groupid]
-            yield from self._messages[offset:]
+            self._offsets[topic][groupid] += 1
+            yield from self._messages[topic][offset:]
         except IndexError:
             pass
-        else:
-            self._offsets[topic][groupid] = len(self._messages[topic]) - 1
 
 
 @pytest.fixture(scope="session")
@@ -215,8 +219,24 @@ def mock_broker():
 @pytest.fixture(scope="session")
 def mock_producer():
     def _mock_producer(mock_broker, topic):
-        producer = MagicMock(spec=Producer)
-        producer.write.side_effect = partialmethod(mock_broker.write, topic=topic)
+        class ProducerBrokerWrapper:
+            def __init__(self, broker, topic):
+                self.broker = broker
+                self.topic = topic
+
+            def write(self, msg):
+                self.broker.write(self.topic, msg)
+
+            def close(self):
+                pass
+
+            def __enter__(self):
+                return self
+
+            def __exit__(self, *exc):
+                pass
+
+        producer = ProducerBrokerWrapper(mock_broker, topic)
         return producer
 
     return _mock_producer
@@ -224,14 +244,39 @@ def mock_producer():
 
 @pytest.fixture(scope="session")
 def mock_consumer():
-    def _mock_consumer(mock_broker, topic, group_id, start_at):
-        consumer = MagicMock(spec=Consumer)
-        consumer.stream.side_effect = partialmethod(
-            mock_broker.read,
-            groupid=group_id,
-            topic=topic,
-            start_at=start_at,
-        )
+    def _mock_consumer(mock_broker, topic, group_id, start_at=StartPosition.EARLIEST):
+        class ConsumerBrokerWrapper:
+            def __init__(self, broker, topic, group_id, start_at):
+                self.broker = broker
+                self.topic = topic
+                self.group_id = group_id
+                self.start_at = start_at
+
+            def subscribe(self, topic):
+                # TODO: Support multiple topics?
+                assert topic == self.topic
+
+            def stream(self, *args, **kwargs):
+                class Message:
+                    def __init__(self, value):
+                        self._value = value
+
+                    def value(self):
+                        return self._value
+
+                for message in self.broker.read(self.topic, self.group_id, self.start_at, **kwargs):
+                    yield Message(message)
+
+            def close(self):
+                pass
+
+            def __enter__(self):
+                return self
+
+            def __exit__(self, *exc):
+                pass
+
+        consumer = ConsumerBrokerWrapper(mock_broker, topic, group_id, start_at)
         return consumer
 
     return _mock_consumer
@@ -318,3 +363,52 @@ def message_parameters_dict():
             "model_text": MESSAGE_BLOB,
         },
     }
+
+
+@contextmanager
+def temp_environ(**vars):
+    """
+    A simple context manager for temporarily setting environment variables
+
+    Kwargs:
+        variables to be set and their values
+
+    Returns:
+        None
+    """
+    from os import environ
+    original = dict(environ)
+    os.environ.update(vars)
+    try:
+        yield  # no value needed
+    finally:
+        # restore original data
+        os.environ.clear()
+        os.environ.update(original)
+
+
+@contextmanager
+def temp_config(tmpdir, data, perms=stat.S_IRUSR | stat.S_IWUSR):
+    """
+    A context manager which creates a temporary config file with specified data and permissions
+
+    Args:
+        data: the data to be written to the file
+        perms: the permissions which should be set on the file.
+            The default value is to use the standard, safe permissions
+
+    Returns:
+        The path to the config directory for hop to use this config file, as a string
+    """
+
+    config_path = f"{tmpdir}/hop/config.toml"
+    os.makedirs(os.path.dirname(config_path), exist_ok=True)
+    config_file = open(config_path, mode='w')
+    os.chmod(config_path, perms)
+    config_file.write(data)
+    config_file.close()
+    try:
+        yield str(tmpdir)
+    finally:
+        # remove file
+        os.remove(config_path)
